@@ -54,6 +54,12 @@ end)
 let assert_equal_string ~msg =
   assert_equal ~printer:(fun x -> x) ~msg:msg
 
+let assert_equal_optional_int ~msg =
+  assert_equal
+  ~printer:(function
+    | Some x -> Printf.sprintf "Some(%d)" x
+    | None -> "None")
+  ~msg:msg
 
 module DiffSetFilename =
   OUnitDiff.SetMake
@@ -484,6 +490,14 @@ let test_fileutil =
     ["Test" >::
      (fun test_ctxt ->
         let tmp_dir = bracket_tmpdir test_ctxt in
+        let () =
+          if Sys.os_type <> "Win32" then begin
+            (* On FreeBSD the default group for a newly created file is
+               the group of the directory. This will result in having a
+               group to be "wheel" on FreeBSD. *)
+            Unix.chown tmp_dir (-1) (Unix.getegid ())
+          end
+        in
         let file_test =
           let fn, chn = bracket_tmpfile test_ctxt in
             output_string chn "foo";
@@ -494,8 +508,10 @@ let test_fileutil =
           non_fatal test_ctxt
             (fun _ ->
                assert_bool
-                 ("Test "^stest^" on "^file)
-                 (res = (test expr file)))
+                (Printf.sprintf
+                  "Test %s on file %s (group ID %d, current group ID %d)"
+                  stest file (Unix.stat file).st_gid (Unix.getegid ()))
+                (res = (test expr file)))
         in
           non_fatal_test file_test ("Size_not_null",   Size_not_null, true);
           List.iter
@@ -1288,6 +1304,50 @@ let test_fileutil =
        ()
     );
 
+    "Cmp" >:::
+    [
+      "same file has no differences" >::
+        (fun test_ctxt ->
+          let tmp_dir = bracket_tmpdir test_ctxt in
+          let file1 = make_filename [tmp_dir; "file1.txt"] in
+          let () =
+            let fd = open_out file1 in
+            output_string fd "Content of file1.";
+            close_out fd
+          in
+          assert_equal_optional_int ~msg:"" None (cmp file1 file1));
+
+      "two identical files have no differences" >::
+        (fun test_ctxt ->
+          let tmp_dir = bracket_tmpdir test_ctxt in
+          let file1 = make_filename [tmp_dir; "file1.txt"] in
+          let file2 = make_filename [tmp_dir; "file2.txt"] in
+          let () =
+            let fd = open_out file1 in
+            output_string fd "Content of file1.";
+            close_out fd
+          in
+          cp [file1] file2;
+          assert_equal_optional_int ~msg:"" None (cmp file1 file2));
+
+      "two different files have differences" >::
+        (fun test_ctxt ->
+          let tmp_dir = bracket_tmpdir test_ctxt in
+          let file1 = make_filename [tmp_dir; "file1.txt"] in
+          let file2 = make_filename [tmp_dir; "file2.txt"] in
+          let () =
+            let fd = open_out file1 in
+            output_string fd "Content of file1.";
+            close_out fd
+          in
+          let () =
+            let fd = open_out file2 in
+            output_string fd "Content of file2.";
+            close_out fd
+          in
+          assert_equal_optional_int ~msg:"" (Some 15) (cmp file1 file2));
+    ];
+
     "Size" >:::
     [
 
